/*
* Atm.cpp
*
* Created: 09/06/2015 18:14:47
* Author: Paul Soulsby
*/


#include "Atm.h"

// default constructor
Atm::Atm() : engine_(AtmEngine::getInstance()), hardware_(AtmHardware::getInstance())
{
	engine_.construct(this);
	hardware_.construct(this);
} //Atm

// default destructor
Atm::~Atm()
{
} //~Atm

void Atm::initialize()
{
	engine_.initialize();
	hardware_.getRotEncoder(AtmHardware::FUNCTION).setContinuous(true);
	hardware_.beginMidi(MIDI_UBRR);
	engine_.getMidiPtr()->setChannel(hardware_.readMidiChannel());
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::YELLOW);
		hardware_.getLedSwitch(AtmHardware::VALUE).setColour(LedRgb::OFF);
		hardware_.getLedSwitch(AtmHardware::BANK).setColour(LedRgb::OFF);
		hardware_.getLedCircular(AtmHardware::VALUE).setState(0);
		hardware_.getRotEncoder(AtmHardware::FUNCTION).setValue((char)engine_.getMidiPtr()->getChannel());
	}
	else
	{
		for(unsigned char i=0;i<2;++i)
		{
			hardware_.getLedSwitch(i).flash(4,LED_FLASH_TICKS,LED_FLASH_TICKS,LedRgb::RED,LedRgb::GREEN,true);
		}
	}
}

void Atm::poll(unsigned char ticksPassed)
{
	hardware_.pollMidi();
	hardware_.pollAnlControls(ticksPassed);
	hardware_.pollSwitches(ticksPassed);
	hardware_.pollRotEncoders(ticksPassed);
	hardware_.refreshFlash(ticksPassed);
	hardware_.refreshLeds();
	engine_.poll(ticksPassed);
	engine_.refreshSysex();
}

bool Atm::isShiftHold(AtmEngine::Func func)
{
	unsigned char i = func>>3;
	return (bool)bitRead(IS_SHIFT_HOLD[i],func-(i*8));
}

bool Atm::isFuncFill(AtmEngine::Func func) const
{
	unsigned char i = func>>3;
	return (bool)bitRead(IS_FUNC_FILL[i],func-(i*8));
}

//***********engine events*********************
void Atm::engineFunctionChanged(unsigned char func, unsigned char val, bool opt)
{
	bool col;
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		hardware_.getLedCircular(AtmHardware::FUNCTION).select(val);
	}
	else
	{
		hardware_.getRotEncoder(AtmHardware::VALUE).setValue((char)val);
		if(isFuncFill((AtmEngine::Func)func)==true)
		{
			hardware_.getRotEncoder(AtmHardware::VALUE).setContinuous(false);
			hardware_.getLedCircular(AtmHardware::VALUE).fill(val);
		}
		else
		{
			hardware_.getRotEncoder(AtmHardware::VALUE).setContinuous(true);
			hardware_.getLedCircular(AtmHardware::VALUE).select(val);
		}
		hardware_.getLedCircular(AtmHardware::FUNCTION).select((unsigned char)func);
		if(func==AtmEngine::FUNC_AENVD)
		{
			col = engine_.getMidiPtr()->getSysexEnable();
		}
		else
		{
			col = opt;
		}
		if(col==true)
		{
			hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::GREEN);
		}
		else
		{
			hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::RED);
		}
	}

}
void Atm::engineBankChanged(unsigned char bank)
{
	hardware_.getLedSwitch(AtmHardware::BANK).flashStop();
	if(bank==HIGH)
	{
		hardware_.getLedSwitch(AtmHardware::BANK).setColour(LedRgb::GREEN);
	}
	else
	{
		hardware_.getLedSwitch(AtmHardware::BANK).setColour(LedRgb::RED);
	}
}
void Atm::engineMidiTransmit(unsigned char data)
{
	hardware_.writeMidi(data);
}

//**************************hardware events************************************
void Atm::hardwareAnalogueControlChanged(unsigned char control, unsigned char newValue)
{
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		return;
	}
	engine_.getPatchPtr()->setCtrlValue(engine_.getBank(),control,newValue);
}

void Atm::hardwareSwitchChanged(unsigned char switch_, unsigned char newValue)
{
	bool opt;
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		return;
	}
	if(switch_==AtmHardware::FUNCTION)
	{
		if(isShiftHold(engine_.getFunction())==true)
		{
			if(newValue==HIGH)
			{
				hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::GREEN);
			}
			else
			{
				hardware_.getLedSwitch(AtmHardware::FUNCTION).flashStop();
				hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::RED);
				if(hardware_.getSwitch(switch_).getHoldTime()>AtmHardware::HOLD_EVENT_TICKS)
				{
					switch(engine_.getFunction())
					{
						case AtmEngine::FUNC_AENVA:
						engine_.writeSysexPatch(engine_.getPatchPtr()->getFunctionValue(AtmEngine::FUNC_MEM));
						break;
						case AtmEngine::FUNC_AENVR:
						engine_.writeSysexMemDump();
						break;
						case AtmEngine::FUNC_MEM:
						engine_.getPatchPtr()->writePatch(engine_.getPatchPtr()->getFunctionValue(AtmEngine::FUNC_MEM));
						break;
					}
				}
				else
				{
					switch(engine_.getFunction())
					{
						case AtmEngine::FUNC_MEM:
						engine_.getPatchPtr()->readPatch(engine_.getPatchPtr()->getFunctionValue(AtmEngine::FUNC_MEM));
						break;
					}
				}
			}
		}
		else if(newValue==HIGH)
		{
			if(engine_.getFunction()==AtmEngine::FUNC_AENVD)
			{
				opt = !engine_.getMidiPtr()->getSysexEnable();
				engine_.getMidiPtr()->setSysexEnable(opt);
			}
			else
			{
				opt = !engine_.getPatchPtr()->getOptionValue(engine_.getFunction());
				engine_.getPatchPtr()->setOptionValue(engine_.getFunction(),opt);
			}
			if(opt==true)
			{
				hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::GREEN);
			}
			else
			{
				hardware_.getLedSwitch(AtmHardware::FUNCTION).setColour(LedRgb::RED);
			}
		}
	}

	
	if(switch_==AtmHardware::VALUE)
	{
		if(newValue==HIGH)
		{
			hardware_.getRotEncoder(AtmHardware::VALUE).setUpdateVal(false);
			hardware_.getLedSwitch(AtmHardware::VALUE).setColour(LedRgb::GREEN);
			engine_.midiNoteOnReceived(testNote_,127);
		}
		else
		{
			hardware_.getRotEncoder(AtmHardware::VALUE).setUpdateVal(true);
			hardware_.getLedSwitch(AtmHardware::VALUE).setColour(LedRgb::RED);
			engine_.midiNoteOffReceived(testNote_);
		}
	}
	
	if(switch_==AtmHardware::BANK)
	{
		if(newValue==HIGH)
		{
			engine_.setBank(HIGH-engine_.getBank());
		}
	}
}
void Atm::hardwareSwitchHeld(unsigned char switch_)
{
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		return;
	}
	if(switch_==AtmHardware::FUNCTION && isShiftHold(engine_.getFunction())==true)
	{
		hardware_.getLedSwitch(AtmHardware::FUNCTION).flash(8,LED_FLASH_TICKS,LED_FLASH_TICKS,LedRgb::GREEN,LedRgb::RED,true);
	}
	if(switch_==AtmHardware::BANK)
	{
		if(engine_.getBank()==LOW)
		{
			hardware_.getLedSwitch(AtmHardware::BANK).flash(8,LED_FLASH_TICKS,LED_FLASH_TICKS,LedRgb::RED,LedRgb::YELLOW,true);
		}
		else
		{
			hardware_.getLedSwitch(AtmHardware::BANK).flash(8,LED_FLASH_TICKS,LED_FLASH_TICKS,LedRgb::GREEN,LedRgb::YELLOW,true);
		}
		engine_.getPatchPtr()->easterEgg(engine_.getMasterClock().getOutput());
	}
}
void Atm::hardwareRotaryEncoderChanged(unsigned char rotary, unsigned char newValue, bool clockwise)
{
	if(hardware_.getMidiChannelSelectMode()==true)
	{
		if(rotary==AtmHardware::FUNCTION)
		{
			hardware_.writeMidiChannel(newValue);
			engine_.getMidiPtr()->setChannel(newValue);
		}
	}
	else
	{
		if(rotary==AtmHardware::FUNCTION)
		{
			engine_.setFunction((AtmEngine::Func)newValue);
		}

		if(rotary==AtmHardware::VALUE)
		{
			if(hardware_.getSwitch(AtmHardware::VALUE).getState()==HIGH)
			{
				if(testNote_<127 && testNote_>0)
				{
					engine_.midiNoteOffReceived(testNote_);
					if(clockwise==true)
					{
						testNote_++;
					}
					else
					{
						testNote_--;
					}
					engine_.midiNoteOnReceived(testNote_,127);
				}
			}
			else
			{
				engine_.getPatchPtr()->setFunctionValue(engine_.getFunction(),newValue);
			}
		}
	}

}
void Atm::hardwareMidiReceived(unsigned char data)
{
	engine_.getMidiPtr()->read(data);
}
void Atm::hardwareMidiError(unsigned char errorType)
{
	hardware_.getLedSwitch(AtmHardware::VALUE).setColour((LedRgb::LedRgbColour)(3-(unsigned char)hardware_.getLedSwitch(AtmHardware::VALUE).getColour()));
	hardware_.getLedCircular(AtmHardware::VALUE).fill(errorType);
}