/* 
* SS_Atmegatron_HardwareTester.cpp
*
* Created: 10/06/2015 10:09:43
* Author: Paul Soulsby
*/


#include "SS_Atmegatron_HardwareTester.h"

// default constructor
SS_Atmegatron_HardwareTester::SS_Atmegatron_HardwareTester()
{
	hardware_ = new SS_Atmegatron_Hardware(this);
	audio_ = new SS_Atmegatron_Audio(kWavelen);
} //SS_Atmegatron_HardwareTester

// default destructor
SS_Atmegatron_HardwareTester::~SS_Atmegatron_HardwareTester()
{
} //~SS_Atmegatron_HardwareTester

void SS_Atmegatron_HardwareTester::Init()
{
	audio_->Init();
	PlaySound(false);
}
void SS_Atmegatron_HardwareTester::PlaySound(bool way)
{
	SS_Wavetable wavetable_(kWavelen);
	for(unsigned char i=0;i<kWavelen;++i)
	{
		if(way==true)
		{
			wavetable_.set_sample(i,sine_wave[i]);
		}
		else
		{
			wavetable_.set_sample(i,0);
		}
	}
	audio_->PasteWavetable(wavetable_);	
}
void SS_Atmegatron_HardwareTester::Tick()
{
	unsigned char i;
	for(i=0;i<2;++i)
	{
		if(circ_ticking[i]==true)
		{
			hardware_->LEDcircular[i].Select(tick);
		}
	}
	for(i=0;i<3;++i)
	{
		if(sw_ticking[i]==true)
		{
			if(tick%2==0)
			{
				hardware_->LEDswitch[i].set_led(SS_LEDrgb::RED);
			}
			else
			{
				hardware_->LEDswitch[i].set_led(SS_LEDrgb::GREEN);
			}
		}
	}
	tick++;
	if(tick>15)
	{
		tick = 0;
	}
}

void SS_Atmegatron_HardwareTester::Poll(unsigned char ticks_passed)
{
	static bool last_moving[6];
	hardware_->PollSwitches(ticks_passed);
	hardware_->PollRotary(ticks_passed);
	hardware_->PollSwitches(ticks_passed);
	hardware_->PollCtrl(ticks_passed);
	hardware_->RefreshLEDs();

	if(circ_ticking[1]==false)
	{
		for(unsigned char i=0;i<6;++i)
		{
			if(hardware_->ctrl[i]->get_moving()!=last_moving[i])
			{
				last_moving[i] = hardware_->ctrl[i]->get_moving();
				if(last_moving[i]==true)
				{
					hardware_->LEDcircular[1].set_led(i,true);
				}
				else
				{
					hardware_->LEDcircular[1].set_led(i,false);
				}
			}

		}		
	}
}

void SS_Atmegatron_HardwareTester::ControlChanged(unsigned char control, unsigned char new_val)
{
	circ_ticking[0] = false;
	circ_ticking[1] = false;
	hardware_->LEDcircular[0].Fill(new_val>>4);
	audio_->set_sample_freq((unsigned long)new_val*16*kWavelen);
}

void SS_Atmegatron_HardwareTester::SwitchChanged(unsigned char sw, unsigned char new_val)
{
	sw_ticking[sw] = false;
	if(new_val==HIGH)
	{
		hardware_->LEDswitch[sw].set_led(SS_LEDrgb::GREEN);
		PlaySound(true);
	}
	else
	{
		hardware_->LEDswitch[sw].set_led(SS_LEDrgb::RED);
		PlaySound(false);
	}
}
void SS_Atmegatron_HardwareTester::SwitchHoldEvent(unsigned char sw)
{
	sw_ticking[sw] = false;
	hardware_->LEDswitch[sw].set_led(SS_LEDrgb::YELLOW);
}
void SS_Atmegatron_HardwareTester::RotaryChanged(unsigned char rotary, unsigned char new_val, bool clockwise)
{
	circ_ticking[rotary] = false;
	hardware_->LEDcircular[rotary].Select(new_val);	
}
void SS_Atmegatron_HardwareTester::MidiRX(unsigned char data)
{

}