/* 
* AtmOscillator.cpp
*
* Created: 05/05/2015 17:17:21
* Author: Paul Soulsby
*/


#include "AtmOscillator.h"

extern void writeMemory(const void* data, void* startAddr, size_t size);
extern void readMemory(void* data, const void* startAddr, size_t size);

// default constructor
AtmOscillator::AtmOscillator(unsigned char waveLength, unsigned int userWaveStartAddress)
{
	waveLength_ = waveLength;
	userWavetable_ = new Wavetable(waveLength_);
	userWaveStartAddress_ = userWaveStartAddress;
	readUserWave(0);
} //AtmOscillator

// default destructor
AtmOscillator::~AtmOscillator()
{
	if(userWavetable_ != NULL)
	{
		delete userWavetable_;
	}
} //~AtmOscillator

void AtmOscillator::copyWavetable(Wavetable& destWavetable)
{
	unsigned char i;
	if(userMode_==true)
	{
		if(userWavetable_ != NULL)
		{
			destWavetable = *userWavetable_;
		}
	}
	else
	{
		if(table_==15 && bank_==0)
		{
			for(i=0;i<waveLength_;++i)
			{
				destWavetable.setSample(i,rand()>>7);
			}
		}
		else
		{
			for(i=0;i<waveLength_;++i)
			{
				destWavetable.setSample(i,pgm_read_byte(&(oscWavetable[bank_][table_][i])));
			}
		}
	}
}
void AtmOscillator::setTable(unsigned char newTable)
{
	table_ = newTable;
	if(userMode_==true)
	{
		readUserWave(newTable);
	}
}
void AtmOscillator::setUserMode(bool newUserMode)
{
	userMode_ = newUserMode;
	if(userMode_==true)
	{
		readUserWave(table_);
	}
}
void AtmOscillator::setUserWavetableSample(unsigned char index,char newSample)
{
	userWavetable_->setSample(index,newSample);
}
char AtmOscillator::getUserWavetableSample(unsigned char index)
{
	return userWavetable_->getSample(index);
}
void AtmOscillator::writeUserWave(unsigned char waveNum)
{
	unsigned int addr  = ((unsigned int)waveNum * waveLength_) + userWaveStartAddress_;     
	unsigned char data[waveLength_];                                            
	for (unsigned char i=0;i<waveLength_;++i)
	{
		data[i] = userWavetable_->getSample(i);
	}
	writeMemory((const void*)data, (void*)addr, sizeof(data));
}
void AtmOscillator::readUserWave(unsigned char waveNum)
{
	unsigned int addr = ((unsigned int)waveNum * waveLength_) + userWaveStartAddress_;
	unsigned char data[waveLength_];
	readMemory((void*)data,(const void*)addr, sizeof(data));
	for (unsigned char i=0;i<waveLength_;++i)
	{
		userWavetable_->setSample(i,(char)data[i]);
	}	
}