/* 
* AtmPatch.cpp
*
* Created: 07/05/2015 10:47:14
* Author: Paul Soulsby
*/


#include "AtmPatch.h"

extern void writeMemory(const void* data, void* startAddr, size_t size);
extern void readMemory(void* data, const void* startAddr, size_t size);
// default constructor
AtmPatch::AtmPatch(AtmPatchBase* base)
{
	if(base!=NULL)
	{
		base_ = base;
	}
} //AtmPatch

// default destructor
AtmPatch::~AtmPatch()
{
	if(base_!=NULL)
	{
		delete base_;
	}
} //~AtmPatch

void AtmPatch::setFunctionValue(unsigned char func, unsigned char newValue)
{
	unsigned char i = func>>1;
	funcValue_c_[i] = compressFourBit(funcValue_c_[i],newValue,(bool)(func%2));
	if(base_!=NULL)
	{
		base_->patchValueChanged(func,newValue);
	}
}

unsigned char AtmPatch::getFunctionValue(unsigned char func)
{
	unsigned char i = func>>1;
	return uncompressFourBit(funcValue_c_[i],(bool)(func%2));
}
void AtmPatch::setOptionValue(unsigned char func, bool newValue)
{
	unsigned char i = func>>3;
	unsigned char b = func - (8*i);
	bitWrite(optionValue_c_[i],b,newValue);
	if(base_!=NULL)
	{
		base_->patchOptionChanged(func,newValue);
	}
}
bool AtmPatch::getOptionValue(unsigned char func)
{
	unsigned char i = func>>3;
	unsigned char b = func - (8*i);
	return (bool)bitRead(optionValue_c_[i],b);
}
void AtmPatch::setCtrlValue(unsigned char bank, unsigned char ctrl, unsigned char newValue)
{
	ctrlValue_[bank][ctrl] = newValue;
	if(base_!=NULL)
	{
		base_->patchCtrlChanged(bank,ctrl,newValue);
	}
}
unsigned char AtmPatch::getCtrlValue(unsigned char bank, unsigned char ctrl)
{
	return ctrlValue_[bank][ctrl];
}
void AtmPatch::writePatch(unsigned char patchNum)
{
	unsigned int startAddr = (unsigned int)PATCH_SIZE * patchNum;
	unsigned int addr = 0;
	unsigned char i,j;
	unsigned char data[PATCH_SIZE];
	for(i=0;i<8;++i)
	{
		data[addr] = funcValue_c_[i];
		addr++;		
	}
	for(i=0;i<2;++i)
	{
		for(j=0;j<6;++j)
		{
			data[addr] = ctrlValue_[i][j];
			addr++;			
		}
	}
	for(i=0;i<2;++i)
	{
		data[addr] = optionValue_c_[i];
		addr++;
	}
	writeMemory((const void*)data, (void*)startAddr, sizeof(data));
}
void AtmPatch::readPatch(unsigned char patchNum)
{
	unsigned int startAddr = (unsigned int)PATCH_SIZE * patchNum;
	unsigned int addr = 0;
	unsigned char i,j;
	unsigned char data[PATCH_SIZE];
	readMemory((void*)data,(const void*)startAddr, sizeof(data));
	for(i=0;i<8;++i)
	{
		setFunctionValue(i<<1,uncompressFourBit(data[addr],false));
		setFunctionValue((i<<1)+1,uncompressFourBit(data[addr],true));
		addr++;
	}
	for(i=0;i<2;++i)
	{
		for(j=0;j<6;++j)
		{
			setCtrlValue(i,j,data[addr]);
			addr++;	
		}
	}
	for(i=0;i<2;++i)
	{
		for(j=0;j<8;++j)
		{
			setOptionValue((i*8)+j,(bool)bitRead(data[addr],j));
		}
		addr++;
	}
}
unsigned char AtmPatch::getSysexByte(unsigned char index)
{
	if(index<16)
	{
		return getFunctionValue(index);
	}
	else if(index<22)
	{
		return getCtrlValue(LOW,index-16)>>1;
	}
	else if (index<28)
	{
		return getCtrlValue(HIGH,index-22)>>1;
	}
	else
	{
		return (unsigned char)getOptionValue(index-28);
	}
}
void AtmPatch::setSysexByte(unsigned char index, unsigned char newValue)
{
	if(index<16)
	{
		setFunctionValue(index,newValue);
	}
	else if (index<22)
	{
		setCtrlValue(LOW,index-16,newValue<<1);
	}
	else if (index<28)
	{
		setCtrlValue(HIGH,index-22,newValue<<1);
	}
	else
	{
		setOptionValue(index-28,(bool)newValue);
	}
}
void AtmPatch::easterEgg(unsigned int seed)
{
	unsigned char i,j;
	srand(seed);
	for(i=0;i<15;++i)  //don't touch patch num
	{
		setFunctionValue(i,rand()>>11);
		if(i<5 || i>8)
		{
			setOptionValue(i,(bool)(rand()>>14));
		}
	}
	for(i=0;i<2;++i)
	{
		for(j=0;j<6;++j)
		{
			setCtrlValue(i,j,rand()>>7);
		}
	}
}