/* 
* BiquadFilter.h
*
* Created: 16/05/2015 18:47:31
* Author: Paul Soulsby
*/


#ifndef __BIQUADFILTER_H__
#define __BIQUADFILTER_H__

#include <math.h>
#include <stdlib.h>
#include "Wavetable.h"
#include "SsHelpers.h"

class BiquadFilter
{
//variables
public:
	enum FiltType : unsigned char
	{
		FILT_OFF,
		FILT_LPF,
		FILT_HPF,
		FILT_BPF,
		FILT_NOTCH,
		FILT_PEAK10,
		FILT_PEAK30,
		FILT_PEAK100,
		FILT_LS10,
		FILT_LS30,
		FILT_HS10,
		FILT_HS30,
		FILT_BUTLPF,
		FILT_BUTHPF,
		FILT_BESLPF,
		FILT_BESHPF
	};
protected:
private:
	static const float MAX_ENV_LFO = 1.386294361119891;
	static const unsigned char MULT_ENV_LFO = 64;
	static const unsigned char BS_ENV_LFO = 6;
	static const float MIN_Q = 0.5;
	static const float MULT_Q = 0.076470588235294;
	unsigned char fc_ = 120;
	unsigned char q_;
	FiltType type_ = FILT_LPF;
	bool gainAdj_;
	unsigned char envAmount_;
	float envAmountF_;
	unsigned char lfoAmount_;
	float lfoAmountF_;
	float a_;
	float a0_ = 1;
	float a1_;
	float a2_;
	float b0_;
	float b1_;
	float b2_;
	float bi1_;
	float bi2_;
	float bo1_;
	float bo2_;
//functions
public:
	BiquadFilter();
	~BiquadFilter();
	void setFc(unsigned char newFc){fc_ = newFc;}
	unsigned char getFc(){return fc_;}
	void setQ(unsigned char newQ){q_ = newQ;}
	unsigned char getQ(){return q_;}
	void setType(FiltType newType);
	FiltType getType(){return type_;}
	void setGainAdj(bool newGainAdj);
	bool getGainAdj(){return gainAdj_;}
	void setEnvAmount(unsigned char newAmount);
	unsigned char getEnvAmount(){return envAmount_;}
	void setLfoAmount(unsigned char newAmount);
	unsigned char getLfoAmount(){return lfoAmount_;}
	void refresh(unsigned long sampleFreq, char lfoOutput, char envOutput);
	void processWavetable(Wavetable& sourceWavetable);
protected:
private:
	BiquadFilter( const BiquadFilter &c );
	BiquadFilter& operator=( const BiquadFilter &c );
	void BiquadCalculator(FiltType type, float fc, float q, float a, float piOverSf);
	float BiquadProcess(float bi0);
}; //BiquadFilter

#endif //__BIQUADFILTER_H__
