/* 
* Flanger.cpp
*
* Created: 30/05/2015 23:37:47
* Author: Paul Soulsby
*/


#include "Flanger.h"

// default constructor
Flanger::Flanger(unsigned char waveLength)
{
	waveLength_ = waveLength;
	wavetable_ = new Wavetable(waveLength_);
	scale_ = 256 / waveLength_;
} //Flanger

// default destructor
Flanger::~Flanger()
{
	if(wavetable_!=NULL)
	{
		delete wavetable_;
	}
} //~Flanger
void Flanger::setLfoAmount(unsigned char newValue)
{
	lfoAmount_ = newValue;
	if(lfoAmount_==0 && envAmount_==0)
	{
		clearBuffer();
	}
}

void Flanger::setEnvAmount(unsigned char newValue)
{
	envAmount_ = newValue;
	if(lfoAmount_==0 && envAmount_==0)
	{
		clearBuffer();
	}	
}

void Flanger::processWavetable(Wavetable& sourceWavetable, char envLevel, char lfoLevel)
{
    unsigned char i;
	unsigned char readpos = 0;
	unsigned char offset = 0;
	static unsigned char writepos = 0;
	int input, output;
	
    if (lfoAmount_>0 || envAmount_>0)
    {		
		unsigned char lfoOffset = ((unsigned int)lfoAmount_+1) * (lfoLevel + 127) >> 8;
		lfoOffset /= scale_;
		unsigned char envOffset = ((unsigned int)envAmount_+1) * abs(envLevel) >> 7;
		envOffset /= scale_;
		if(lfoOffset+envOffset>waveLength_)
		{
			offset = waveLength_;
		}
		else
		{
			offset = lfoOffset + envOffset;
		}
		if(offset==0)
		{
			offset = 1;
		}
		if(offset>writepos)
		{
			readpos = writepos + waveLength_ - offset;
		}
		else
		{
			readpos = writepos - offset;
		} 
		for(i=0;i<sourceWavetable.getWaveLength();++i)
		{
			input = (sourceWavetable.getSample(i)) + (wavetable_->getSample(readpos)>>fbBs_);
			wavetable_->setSample(writepos,constrainInt(input));
			
			output = (sourceWavetable.getSample(i)>>dryBs_) + (wavetable_->getSample(readpos)>>wetBs_);	
			sourceWavetable.setSample(i,constrainInt(output));	
			writepos++;
			if(writepos>=waveLength_)
			{
				writepos = 0;
			}
			readpos++;
			if(readpos>=waveLength_)
			{
				readpos = 0;
			}
		}
		cleared_ = false;
    }	
}
void Flanger::clearBuffer()
{
	if(cleared_==false)
	{
		wavetable_->clearSamples();
		cleared_ = true;
	}
	
}