/*
* Midi.cpp
*
* Created: 26/05/2015 16:56:30
* Author: Paul Soulsby
*/


#include "Midi.h"

// default constructor
Midi::Midi(MidiBase* base, unsigned char sysex_prod_id)
{
	base_ = base;
	sysexProductId_ = sysex_prod_id;
} //Midi

// default destructor
Midi::~Midi()
{
} //~Midi

unsigned char Midi::getClockTicksPassed()
{
	unsigned char ticksPassed = clockTicks_ - clockTicksLast_;
	clockTicksLast_ = clockTicks_;
	return ticksPassed;
}
void Midi::setChannel(unsigned char new_channel)
{
	if(new_channel>15)
	{
		channel_ = 0;
	}
	else
	{
		channel_ = new_channel;
	}
	base_->midiChannelChanged(channel_);
}
void Midi::reset()
{
	for(unsigned char i=0;i<128;++i)
	{
		base_->midiNoteOffReceived(i);
	}
}

void Midi::read(unsigned char in_byte)
{
	unsigned char byte_cnt = 0;
	static unsigned char note_byte=0;
	static unsigned char vel_byte=0;
	static unsigned char status_buffer=0;
	static bool first_byte;
	static unsigned char sysex_rx_mnftr = 255;
	static unsigned char sysex_rx_prodid = 255;

	if (in_byte>=CLOCK)
	{
		switch (in_byte)
		{
			case CLOCK:
			clockTicks_++;
			break;
			case START:
			case CONTINUE:  //currently implemented same as start
			clockTicks_ = 0;
			clockTicksLast_ = 0;
			clockRunning_ = true;
			base_->midiClockStartReceived();
			break;
			case STOP:
			clockRunning_ = false;
			base_->midiClockStopReceived();
			break;
		}
	}
	else if (in_byte>=SYSEX_START)
	{
		status_buffer = 0;
		if (in_byte==SYSEX_START)
		{
			sysexRx_ = true;
			sysexIndex_ = 0;
			sysex_rx_mnftr = 255;
			sysex_rx_prodid = 255;
			if(sysexEnable_==true)
			{
				base_->midiSysexStartReceived();
			}
			
		}
		if (in_byte==SYSEX_STOP)
		{
			sysexRx_ = false;
			if(sysexEnable_==true)
			{
				base_->midiSysexStopReceived();
			}
		}
	}
	else if (in_byte>=NOTE_OFF)
	{
		status_buffer = in_byte;
		first_byte = true;
		note_byte = 0;
		vel_byte = 0;
		sysexRx_ = false;
	}
	else if (sysexRx_==true)
	{
		sysexIndex_++;
		if(sysexIndex_==1)
		{
			sysex_rx_mnftr = in_byte;
		}
		else if(sysexIndex_==2)
		{
			sysex_rx_prodid = in_byte;
		}
		else if (sysex_rx_mnftr==SYSEX_MANUFACTURER && sysex_rx_prodid==sysexProductId_)
		{
			if(sysexEnable_==true)
			{
				base_->midiSysexDataReceived(sysexIndex_-3,in_byte);
			}
		}
	}
	else if (status_buffer!=0)
	{
		if (first_byte == true)
		{
			note_byte = in_byte;
			first_byte = false;
		}
		else
		{
			vel_byte = in_byte;
			if (status_buffer == (NOTE_ON | channel_) && vel_byte > 0)
			{
				base_->midiNoteOnReceived(note_byte,vel_byte);
			}
			else if (status_buffer == (NOTE_OFF | channel_) || (status_buffer == (NOTE_ON | channel_) && vel_byte == 0))
			{
				base_->midiNoteOffReceived(note_byte);
			}
			else if (status_buffer == (PITCH_BEND | channel_))
			{
				if(vel_byte==0)
				{
					base_->midiPitchBendReceived(-127);
				}
				else if (vel_byte==127)
				{
					base_->midiPitchBendReceived(127);
				}
				else
				{
					base_->midiPitchBendReceived((vel_byte-64)<<1);
				}
			}
			else if (status_buffer == (CONTROL_CHANGE | channel_))
			{
				base_->midiControlChangeReceived(note_byte,vel_byte);
			}
			note_byte = 0;
			vel_byte = 0;
			first_byte = true;
		}
	}
}
void Midi::writeSysex(unsigned char* sys_mess, unsigned int size)
{
	base_->midiSysexWrite(SYSEX_START);
	base_->midiSysexWrite(SYSEX_MANUFACTURER);
	base_->midiSysexWrite(sysexProductId_);
	for(unsigned int i=0;i<size;++i)
	{
		base_->midiSysexWrite(sys_mess[i]);
	}	
	base_->midiSysexWrite(SYSEX_STOP);
}
